
library(diversitree)
library(akima)
library(scatterplot3d)

contourSearchPoints<-function(turnover, eps, nrep=10000){
	sd.mult=3
	turnover <- as.numeric(turnover)
	eps <- as.numeric(eps)

	rnorm.bounded<-function(mean=0,sd=1,bound=0) {
		x<-(-Inf)
		while (x<bound) {
			x<-rnorm(1,mean,sd) 
		}

		return(x)
	}

	param1.points<-replicate(n=round(nrep/4),expr=rnorm.bounded(mean=turnover, sd=.5))
	param2.points<-replicate(n=round(nrep/4),expr=rnorm.bounded(mean=eps, sd=.5))

	param1.points<-c(param1.points,replicate(n=round(nrep/4),expr=rnorm.bounded(mean=turnover,sd=sd.mult*.5)))
	param2.points<-c(param2.points,replicate(n=round(nrep/4),expr=rnorm.bounded(mean=eps,sd=sd.mult*.5)))

	xlim1=range(param1.points,0,3)
	ylim1=range(param2.points,0,3)
		
	#I added this in there so that the hinterland does not include values that are not biologically possible:
	if(xlim1[1]<0){
		xlim1[1]=0
	}
	if(ylim1[1]<0){
		ylim1[1]=0
	}
	
	param1.points<-c(param1.points,runif(nrep-round(nrep/2),min=min(xlim1),max=max(xlim1)))
	param2.points<-c(param2.points,runif(nrep-round(nrep/2),min=min(ylim1),max=max(ylim1)))

	#now randomize order, in case we later want to do incremental saving or drawing
	param1.points<-sample(param1.points,size=length(param1.points),replace=FALSE)
	param2.points<-sample(param2.points,size=length(param2.points),replace=FALSE)

	param.points<-cbind(param1.points,param2.points)
	param.points
}


contourSearch<-function(phy, param.points, nrep=10000){
	
	res<-matrix(,nrep,3)
	for(i in 1:nrep){
		
#		cat("Starting", "iteration",i,"\n") 
		
		#have to put into global scope due to how diversitree checks for unknown parameters
		turnoveri<<-param.points[i,1]
		epsi<<-param.points[i,2]
		lambda.p <- turnoveri/(1+epsi) 
		mu.p <- (epsi*turnoveri)/(1+epsi)
		lik <- make.bd(phy)
		loglik <- lik(c(lambda.p,mu.p))
		res[i,1]<-loglik
		res[i,2]<-turnoveri
		res[i,3]<-epsi
		
#		write.table(q, file=file.name,row.name=FALSE, quote=FALSE, col.name=FALSE, sep="\t", append=TRUE)
#		cat("Completed", i, "iterations", "\n") 
	}
	res
}

contourPlot<-function(surfaceData, col, newplot=FALSE,turnover,eps) {
	mydata<-data.frame(x=matrix(surfaceData[,3],ncol=1),y=matrix(surfaceData[,2],ncol=1),z=matrix(surfaceData[,1],ncol=1))
	mydata$z<-(-1)*mydata$z
	mydata$z<-mydata$z-min(mydata$z)
	
	xlab<-"d/b"
	ylab<-"b+d"
	xlim=c(0,1.5)
	ylim=c(0,1.5)
	
	if(newplot==TRUE){
		contour(interp(x= mydata$x,y=mydata$y,z=mydata$z,xo=seq(min(mydata$x), max(mydata$x),length = 400), yo=seq(min(mydata$y), max(mydata$y),length = 400),duplicate="strip"),levels=c(2),asp=1,col=col,xlim=xlim,ylim=ylim,axes=FALSE,drawlabels=FALSE,xlab="",ylab="",bty="n")
	}
	else{
		contour(interp(x= mydata$x,y=mydata$y,z=mydata$z,xo=seq(min(mydata$x), max(mydata$x),length = 400), yo=seq(min(mydata$y), max(mydata$y),length = 400),duplicate="strip"),levels=c(2),asp=1,col=col,xlim=xlim,ylim=ylim,axes=FALSE,drawlabels=FALSE,xlab=xlab,ylab=ylab,bty="n")
		par(tck=.01)
		axis(2, at = seq(0, 1.5, by = .5), las =1, lwd=1, labels=TRUE, mgp=c(.75,.5,0))
		axis(1, at = seq(0, 1.5, by = .5), las =1, lwd=1, labels=TRUE, mgp=c(.75,.5,0))
		axis(2, at = seq(0, 1.5, by = .25), las =1, lwd=1, labels=FALSE, mgp=c(.75,.5,0))
		axis(1, at = seq(0, 1.5, by = .25), las =1, lwd=1, labels=FALSE, mgp=c(.75,.5,0))
	}
	points(x=eps, y=turnover,pch=19,cex=.5)
#	if(substr(names(surfaceData)[2],1,1)==substr(names(surfaceData)[3],1,1)) {
#		lines(x=c(0,max(mydata[,2:3])),y=c(0,max(mydata[,2:3])),lty="dotted")
#	}
	
}

contourPlotSet<-function(phy,turnover,eps,newplot,col){

	cat("Starting", "tree",1,"\n") 
	surfDat<-contourSearchPoints(turnover=turnover,eps=eps)
	fee<-contourSearch(phy[[1]],param.points=surfDat)
#	try(contourPlot(surfaceData=fee, col="gray", newplot=newplot))
	tmp<-fee[,1]
	for(i in 2:length(phy)){
		cat("Starting", "tree", i,"\n")
		foo<-contourSearch(phy[[i]],param.points=surfDat)
#		par(new=TRUE)
#		try(contourPlot(surfaceData=foo, col="gray", newplot=newplot))
		tmp<-cbind(tmp,foo[,1])
	}
#	par(new=TRUE)
	kk<-rowMeans(tmp)
	tmp<-cbind(kk,fee[,2:3])
	try(contourPlot(surfaceData=tmp, col=col, newplot=newplot,turnover=turnover,eps=eps))
}

pdf("e00.contour.pdf")
par(mfcol=c(6,1), mar=c(.25,.25,0.25,0.25))
par(mfcol=c(2,2),mar=c(4,4,0.5,0.5), oma=c(1.5,2,1,1))

#PanelA:
small25<-read.tree("e00s00.25.trees")
contourPlotSet(small25, turnover=0.05051264,eps=0.00,newplot=FALSE,col=gray(.8))
par(new=TRUE)
small100<-read.tree("e00s00.100.trees")
pp<-contourPlotSet(small100,turnover=0.0782366,eps=0.00,newplot=TRUE,col=gray(.4))

#PanelB:
small25<-read.tree("e00s06.25.trees")
contourPlotSet(small25, turnover=0.05051264,eps=0.00,newplot=FALSE,col=gray(.8))
par(new=TRUE)
small100<-read.tree("e00s06.100.trees")
contourPlotSet(small100,turnover=0.0782366,eps=0.00,newplot=TRUE,col=gray(.4))

dev.off()


pdf("e25.contour.pdf")
par(mfcol=c(6,1), mar=c(.25,.25,0.25,0.25))
par(mfcol=c(2,2),mar=c(4,4,0.5,0.5), oma=c(1.5,2,1,1))

#PanelA:
small25<-read.tree("e25s00.25.trees")
contourPlotSet(small25, turnover=(0.06587368+(0.06587368*.25)),eps=0.25,newplot=FALSE,col=gray(.8))
par(new=TRUE)
small100<-read.tree("e25s00.100.trees")
pp<-contourPlotSet(small100,turnover=(0.1026514+(0.1026514*.25)),eps=0.25,newplot=TRUE,col=gray(.4))

#PanelB:
small25<-read.tree("e25s06.25.trees")
contourPlotSet(small25, turnover=(0.06587368+(0.06587368*.25)),eps=0.25,newplot=FALSE,col=gray(.8))
par(new=TRUE)
small100<-read.tree("e25s06.100.trees")
contourPlotSet(small100,turnover=(0.1026514+(0.1026514*.25)),eps=0.25,newplot=TRUE,col=gray(.4))

dev.off()


pdf("e50.contour.pdf")
par(mfcol=c(6,1), mar=c(.25,.25,0.25,0.25))
par(mfcol=c(2,2),mar=c(4,4,0.5,0.5), oma=c(1.5,2,1,1))

#PanelA:
small25<-read.tree("e50s00.25.trees")
contourPlotSet(small25, turnover=(0.09088999+(0.09088999*.5)),eps=0.50,newplot=FALSE,col=gray(.8))
par(new=TRUE)
small100<-read.tree("e50s00.100.trees")
pp<-contourPlotSet(small100,turnover=(0.1453241+(0.1453241*.5)),eps=0.50,newplot=TRUE,col=gray(.4))

#PanelB:
small25<-read.tree("e50s06.25.trees")
contourPlotSet(small25, turnover=(0.09088999+(0.09088999*.5)),eps=0.50,newplot=FALSE,col=gray(.8))
par(new=TRUE)
small100<-read.tree("e50s06.100.trees")
contourPlotSet(small100,turnover=(0.1453241+(0.1453241*.5)),eps=0.50,newplot=TRUE,col=gray(.4))

dev.off()


pdf("e75.contour.pdf")
par(mfcol=c(6,1), mar=c(.25,.25,0.25,0.25))
par(mfcol=c(2,2),mar=c(4,4,0.5,0.5), oma=c(1.5,2,1,1))

#PanelA:
small25<-read.tree("e75s00.25.trees")
contourPlotSet(small25, turnover=(0.1447764+(0.1447764*.75)),eps=0.75,newplot=FALSE,col=gray(.8))
par(new=TRUE)
small100<-read.tree("e75s00.100.trees")
pp<-contourPlotSet(small100,turnover=(0.2491702+(0.2491702*.75)),eps=0.75,newplot=TRUE,col=gray(.4))

#PanelB:
small25<-read.tree("e75s06.25.trees")
contourPlotSet(small25, turnover=(0.1447764+(0.1447764*.75)),eps=0.75,newplot=FALSE,col=gray(.8))
par(new=TRUE)
small100<-read.tree("e75s06.100.trees")
contourPlotSet(small100,turnover=(0.2491702+(0.2491702*.75)),eps=0.75,newplot=TRUE,col=gray(.4))

dev.off()


