setwd("~/Dropbox/CollabBeaulieu/estimatesForRaboFig4")
library(ape)

EstimatesIgnoringSubsampling <- function(sampling.f, birth.true, death.true) {
	#uses equations from Stadler, Syst bio, 2013 "How can we improve accuracy of macroevoltuionary rate estimates?"
	#birth.true = birth.est / sampling.f
	birth.est = birth.true * sampling.f
	#death.true = death.est - birth.est * (1 - 1/sampling.f)
	death.est = death.true + birth.est * (1 - 1/sampling.f)
	rates.est <- c(birth.est, death.est)
	names(rates.est) <- c("birth.est", "death.est")
	if(death.true / birth.true < (1 - sampling.f)) {
		return(NA) #correction will not work in this case	
	}
	return(rates.est)
}

ExtinctionFractionIgnoringSubsampling <- function(sampling.f, birth.true, death.true) {
	rates.est <- EstimatesIgnoringSubsampling(sampling.f, birth.true, death.true)
	return(unname(rates.est[2]/rates.est[1]))
}


sampling.vector <- seq(from=0, to=1, length.out=101)

true.fractions <- seq(from=0, to=1, length.out=21)[-1]
pdf(file="BiasFromIgnoringSubsampling.pdf")

plot(c(0,110), c(0,1), type="n", bty="n", xlab="Percentage taxa missing", ylab="Estimated extinction fraction ignoring subsampling")

for (i in sequence(length(true.fractions))) {
	fractions <- sapply(sampling.vector, ExtinctionFractionIgnoringSubsampling, birth.true=1, death.true=true.fractions[i])
	lines(100*(1-sampling.vector), fractions)
	#text(x=1, y=fractions[length(fractions)], true.fractions[i], pos=4, cex=0.8)
	points(x=0, y=true.fractions[i], pch=16)
}
dev.off()

		
#system("open BiasFromIgnoringSubsampling.pdf")
pdf(file="EmpiricalResults.pdf", width=10, height=10) 
par(mfcol=c(2,2),mar=c(4,4,0.5,0.5), oma=c(1.5,2,1,1))
load("Fig4s06.Rsave")
results.df<-results.df[which(results.df$file=="e75s06.500.trees"),] #use only the ones that violate the model
desired.ntax.v<-100*c(1:floor(0.01 * max(results.df$ntaxFinal)))
for(sampleCorrection in 0:1) {
	for (baseonfrac in 0:1) {
		xrange<-range(c(0, max(results.df$ntaxOrig)))
		if(baseonfrac==1) {
			xrange<-c(0,1)
		}
		plot(x=xrange, y=range(c(0,results.df$bd.e / results.df$bd.b, 1)), type="n", axes=FALSE, xlab="", ylab="")
		par(tck=.01)
		axis(2, at = seq(0, 1, by = .2), las =1, lwd=1, labels=TRUE, mgp=c(.75,.5,0))
		if(baseonfrac==1){
			axis(1, at = seq(0, 1, by = .2), las =1, lwd=1, labels=TRUE, mgp=c(.75,.5,0))
		}else{
			axis(1, at = seq(0, 1400, by = 200), las =1, lwd=1, labels=TRUE, mgp=c(.75,.5,0))
		}
		title(xlab=c("Number of taxa", "Sampling fraction")[baseonfrac+1], line=2)
		title(ylab=c(expression(Estimated~epsilon~ignoring~subsampling), expression(Estimated~epsilon~with~subsampling))[sampleCorrection+1], line=2.5)

		for (tree.index in sequence(max(results.df$treeNum))) {
			local.results<-results.df[which(results.df$treeNum==tree.index),]
			local.results<-local.results[which(local.results$sampleCorrection!=c(1,0)[sampleCorrection+1]),]
			local.results<-local.results[order(local.results$ntaxFinal),]
			#print(local.results)
			if(baseonfrac==0) {
				lines(local.results$ntaxFinal, local.results$bd.e / local.results$bd.b, col=rgb(0,0,0,0.2))
				points(local.results$ntaxFinal[length(local.results$ntaxFinal)], (local.results$bd.e / local.results$bd.b)[length(local.results$ntaxFinal)], pch=16)
			} else {
				lines(local.results$ntaxFinal/local.results$ntaxOrig, local.results$bd.e / local.results$bd.b, col=rgb(0,0,0,0.2))
				
			}
		}
		if(baseonfrac==1) {
			if(sampleCorrection == 0) {
				fractions <- sapply(sampling.vector, ExtinctionFractionIgnoringSubsampling, birth.true=1, death.true=.75)
				lines(sampling.vector, fractions, col="red", lwd=2)
			} else {
				abline(h=0.75, col="red", lwd=2)	
			}
		}
	}
}
dev.off()
system("open EmpiricalResults.pdf")
